@@ -61,8 +61,6 @@ static void gst_ffmpegaudenc_base_init (GstFFMpegAudEncClass * klass);
 static void gst_ffmpegaudenc_init (GstFFMpegAudEnc * ffmpegaudenc);
 static void gst_ffmpegaudenc_finalize (GObject * object);
 
-static GstCaps *gst_ffmpegaudenc_getcaps (GstAudioEncoder * encoder,
-    GstCaps * filter);
 static gboolean gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder,
     GstAudioInfo * info);
 static GstFlowReturn gst_ffmpegaudenc_handle_frame (GstAudioEncoder * encoder,
@@ -162,7 +160,6 @@ gst_ffmpegaudenc_class_init (GstFFMpegAudEncClass * klass)
 
   gstaudioencoder_class->start = GST_DEBUG_FUNCPTR (gst_ffmpegaudenc_start);
   gstaudioencoder_class->stop = GST_DEBUG_FUNCPTR (gst_ffmpegaudenc_stop);
-  gstaudioencoder_class->getcaps = GST_DEBUG_FUNCPTR (gst_ffmpegaudenc_getcaps);
   gstaudioencoder_class->flush = GST_DEBUG_FUNCPTR (gst_ffmpegaudenc_flush);
   gstaudioencoder_class->set_format =
       GST_DEBUG_FUNCPTR (gst_ffmpegaudenc_set_format);
@@ -176,9 +173,12 @@ gst_ffmpegaudenc_init (GstFFMpegAudEnc * ffmpegaudenc)
   GstFFMpegAudEncClass *klass =
       (GstFFMpegAudEncClass *) G_OBJECT_GET_CLASS (ffmpegaudenc);
 
+  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (ffmpegaudenc));
+
   /* ffmpeg objects */
   ffmpegaudenc->context = avcodec_alloc_context3 (klass->in_plugin);
   ffmpegaudenc->opened = FALSE;
+  ffmpegaudenc->frame = av_frame_alloc ();
 
   ffmpegaudenc->compliance = FFMPEG_DEFAULT_COMPLIANCE;
 
@@ -191,6 +191,7 @@ gst_ffmpegaudenc_finalize (GObject * object)
   GstFFMpegAudEnc *ffmpegaudenc = (GstFFMpegAudEnc *) object;
 
   /* clean up remaining allocated data */
+  av_frame_free (&ffmpegaudenc->frame);
   gst_ffmpeg_avcodec_close (ffmpegaudenc->context);
   av_free (ffmpegaudenc->context);
 
@@ -236,22 +237,6 @@ gst_ffmpegaudenc_flush (GstAudioEncoder * encoder)
   }
 }
 
-static GstCaps *
-gst_ffmpegaudenc_getcaps (GstAudioEncoder * encoder, GstCaps * filter)
-{
-  GstFFMpegAudEnc *ffmpegaudenc = (GstFFMpegAudEnc *) encoder;
-  GstCaps *caps = NULL;
-
-  GST_DEBUG_OBJECT (ffmpegaudenc, "getting caps");
-
-  /* audio needs no special care */
-  caps = gst_audio_encoder_proxy_getcaps (encoder, NULL, filter);
-
-  GST_DEBUG_OBJECT (ffmpegaudenc, "audio caps, return %" GST_PTR_FORMAT, caps);
-
-  return caps;
-}
-
 static gboolean
 gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
 {
@@ -284,8 +269,9 @@ gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
     ffmpegaudenc->context->bit_rate = ffmpegaudenc->bitrate;
     ffmpegaudenc->context->bit_rate_tolerance = ffmpegaudenc->bitrate;
   } else {
-    GST_INFO_OBJECT (ffmpegaudenc, "Using avcontext default bitrate %d",
-        ffmpegaudenc->context->bit_rate);
+    GST_INFO_OBJECT (ffmpegaudenc,
+        "Using avcontext default bitrate %" G_GINT64_FORMAT,
+        (gint64) ffmpegaudenc->context->bit_rate);
   }
 
   /* RTP payload used for GOB production (for Asterisk) */
@@ -299,7 +285,6 @@ gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
   ffmpegaudenc->context->coder_type = 0;
   ffmpegaudenc->context->context_model = 0;
   ffmpegaudenc->context->scenechange_threshold = 0;
-  ffmpegaudenc->context->inter_threshold = 0;
 
   /* fetch pix_fmt and so on */
   gst_ffmpeg_audioinfo_to_context (info, ffmpegaudenc->context);
@@ -402,125 +387,196 @@ gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
     gst_audio_encoder_set_frame_max (GST_AUDIO_ENCODER (ffmpegaudenc), 0);
   }
 
+  /* Store some tags */
+  {
+    GstTagList *tags = gst_tag_list_new_empty ();
+    const gchar *codec;
+
+    gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_NOMINAL_BITRATE,
+        ffmpegaudenc->context->bit_rate, NULL);
+
+    if ((codec =
+            gst_ffmpeg_get_codecid_longname (ffmpegaudenc->context->codec_id)))
+      gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_AUDIO_CODEC, codec,
+          NULL);
+
+    gst_audio_encoder_merge_tags (encoder, tags, GST_TAG_MERGE_REPLACE);
+    gst_tag_list_unref (tags);
+  }
+
   /* success! */
   ffmpegaudenc->opened = TRUE;
 
   return TRUE;
 }
 
+static void
+gst_ffmpegaudenc_free_avpacket (gpointer pkt)
+{
+  av_packet_unref ((AVPacket *) pkt);
+  g_slice_free (AVPacket, pkt);
+}
+
+typedef struct
+{
+  GstBuffer *buffer;
+  GstMapInfo map;
+
+  guint8 **ext_data_array, *ext_data;
+} BufferInfo;
+
+static void
+buffer_info_free (void *opaque, guint8 * data)
+{
+  BufferInfo *info = opaque;
+
+  if (info->buffer) {
+    gst_buffer_unmap (info->buffer, &info->map);
+    gst_buffer_unref (info->buffer);
+  } else {
+    av_free (info->ext_data);
+    av_free (info->ext_data_array);
+  }
+  g_slice_free (BufferInfo, info);
+}
 
 static GstFlowReturn
 gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
-    guint8 * audio_in, guint in_size, gint * have_data)
+    GstBuffer * buffer, gint * have_data)
 {
   GstAudioEncoder *enc;
   AVCodecContext *ctx;
   gint res;
   GstFlowReturn ret;
   GstAudioInfo *info;
-  AVPacket pkt;
-  AVFrame frame;
+  AVPacket *pkt;
+  AVFrame *frame = ffmpegaudenc->frame;
   gboolean planar;
+  gint nsamples = -1;
 
   enc = GST_AUDIO_ENCODER (ffmpegaudenc);
 
   ctx = ffmpegaudenc->context;
 
-  GST_LOG_OBJECT (ffmpegaudenc, "encoding buffer ");
+  pkt = g_slice_new0 (AVPacket);
 
-  memset (&pkt, 0, sizeof (pkt));
-  memset (&frame, 0, sizeof (frame));
-  avcodec_get_frame_defaults (&frame);
+  if (buffer != NULL) {
+    BufferInfo *buffer_info = g_slice_new0 (BufferInfo);
+    guint8 *audio_in;
+    guint in_size;
 
-  info = gst_audio_encoder_get_audio_info (enc);
-  planar = av_sample_fmt_is_planar (ffmpegaudenc->context->sample_fmt);
+    buffer_info->buffer = buffer;
+    gst_buffer_map (buffer, &buffer_info->map, GST_MAP_READ);
+    audio_in = buffer_info->map.data;
+    in_size = buffer_info->map.size;
 
-  if (planar && info->channels > 1) {
-    gint channels, nsamples;
-    gint i, j;
+    GST_LOG_OBJECT (ffmpegaudenc, "encoding buffer %p size:%u", audio_in,
+        in_size);
 
-    nsamples = frame.nb_samples = in_size / info->bpf;
-    channels = info->channels;
+    info = gst_audio_encoder_get_audio_info (enc);
+    planar = av_sample_fmt_is_planar (ffmpegaudenc->context->sample_fmt);
 
-    if (info->channels > AV_NUM_DATA_POINTERS) {
-      frame.extended_data = g_new (uint8_t *, info->channels);
+    if (planar && info->channels > 1) {
+      gint channels;
+      gint i, j;
+
+      nsamples = frame->nb_samples = in_size / info->bpf;
+      channels = info->channels;
+
+      frame->buf[0] =
+          av_buffer_create (NULL, 0, buffer_info_free, buffer_info, 0);
+
+      if (info->channels > AV_NUM_DATA_POINTERS) {
+        buffer_info->ext_data_array = frame->extended_data =
+            av_malloc_array (info->channels, sizeof (uint8_t *));
+      } else {
+        frame->extended_data = frame->data;
+      }
+
+      buffer_info->ext_data = frame->extended_data[0] = av_malloc (in_size);
+      frame->linesize[0] = in_size / channels;
+      for (i = 1; i < channels; i++)
+        frame->extended_data[i] =
+            frame->extended_data[i - 1] + frame->linesize[0];
+
+      switch (info->finfo->width) {
+        case 8:{
+          const guint8 *idata = (const guint8 *) audio_in;
+
+          for (i = 0; i < nsamples; i++) {
+            for (j = 0; j < channels; j++) {
+              ((guint8 *) frame->extended_data[j])[i] = idata[j];
+            }
+            idata += channels;
+          }
+          break;
+        }
+        case 16:{
+          const guint16 *idata = (const guint16 *) audio_in;
+
+          for (i = 0; i < nsamples; i++) {
+            for (j = 0; j < channels; j++) {
+              ((guint16 *) frame->extended_data[j])[i] = idata[j];
+            }
+            idata += channels;
+          }
+          break;
+        }
+        case 32:{
+          const guint32 *idata = (const guint32 *) audio_in;
+
+          for (i = 0; i < nsamples; i++) {
+            for (j = 0; j < channels; j++) {
+              ((guint32 *) frame->extended_data[j])[i] = idata[j];
+            }
+            idata += channels;
+          }
+
+          break;
+        }
+        case 64:{
+          const guint64 *idata = (const guint64 *) audio_in;
+
+          for (i = 0; i < nsamples; i++) {
+            for (j = 0; j < channels; j++) {
+              ((guint64 *) frame->extended_data[j])[i] = idata[j];
+            }
+            idata += channels;
+          }
+
+          break;
+        }
+        default:
+          g_assert_not_reached ();
+          break;
+      }
+
+      gst_buffer_unmap (buffer, &buffer_info->map);
+      gst_buffer_unref (buffer);
+      buffer_info->buffer = NULL;
     } else {
-      frame.extended_data = frame.data;
+      frame->data[0] = audio_in;
+      frame->extended_data = frame->data;
+      frame->linesize[0] = in_size;
+      frame->nb_samples = nsamples = in_size / info->bpf;
+      frame->buf[0] =
+          av_buffer_create (NULL, 0, buffer_info_free, buffer_info, 0);
     }
 
-    frame.extended_data[0] = g_malloc (in_size);
-    frame.linesize[0] = in_size / channels;
-    for (i = 1; i < channels; i++)
-      frame.extended_data[i] = frame.extended_data[i - 1] + frame.linesize[0];
-
-    switch (info->finfo->width) {
-      case 8:{
-        const guint8 *idata = (const guint8 *) audio_in;
-
-        for (i = 0; i < nsamples; i++) {
-          for (j = 0; j < channels; j++) {
-            ((guint8 *) frame.extended_data[j])[i] = idata[j];
-          }
-          idata += channels;
-        }
-        break;
-      }
-      case 16:{
-        const guint16 *idata = (const guint16 *) audio_in;
-
-        for (i = 0; i < nsamples; i++) {
-          for (j = 0; j < channels; j++) {
-            ((guint16 *) frame.extended_data[j])[i] = idata[j];
-          }
-          idata += channels;
-        }
-        break;
-      }
-      case 32:{
-        const guint32 *idata = (const guint32 *) audio_in;
-
-        for (i = 0; i < nsamples; i++) {
-          for (j = 0; j < channels; j++) {
-            ((guint32 *) frame.extended_data[j])[i] = idata[j];
-          }
-          idata += channels;
-        }
-
-        break;
-      }
-      case 64:{
-        const guint64 *idata = (const guint64 *) audio_in;
-
-        for (i = 0; i < nsamples; i++) {
-          for (j = 0; j < channels; j++) {
-            ((guint64 *) frame.extended_data[j])[i] = idata[j];
-          }
-          idata += channels;
-        }
-
-        break;
-      }
-      default:
-        g_assert_not_reached ();
-        break;
-    }
+    /* we have a frame to feed the encoder */
+    res = avcodec_encode_audio2 (ctx, pkt, frame, have_data);
 
+    av_frame_unref (frame);
   } else {
-    frame.data[0] = audio_in;
-    frame.extended_data = frame.data;
-    frame.linesize[0] = in_size;
-    frame.nb_samples = in_size / info->bpf;
+    GST_LOG_OBJECT (ffmpegaudenc, "draining");
+    /* flushing the encoder */
+    res = avcodec_encode_audio2 (ctx, pkt, NULL, have_data);
   }
 
-  res = avcodec_encode_audio2 (ctx, &pkt, &frame, have_data);
-  if (planar && info->channels > 1)
-    g_free (frame.data[0]);
-  if (frame.extended_data != frame.data)
-    g_free (frame.extended_data);
-
   if (res < 0) {
     char error_str[128] = { 0, };
 
+    g_slice_free (AVPacket, pkt);
     av_strerror (res, error_str, sizeof (error_str));
     GST_ERROR_OBJECT (enc, "Failed to encode buffer: %d - %s", res, error_str);
     return GST_FLOW_OK;
@@ -531,20 +587,24 @@ gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
     GstBuffer *outbuf;
     const AVCodec *codec;
 
-    GST_LOG_OBJECT (ffmpegaudenc, "pushing size %d", pkt.size);
+    GST_LOG_OBJECT (ffmpegaudenc, "pushing size %d", pkt->size);
 
     outbuf =
-        gst_buffer_new_wrapped_full (0, pkt.data, pkt.size, 0, pkt.size,
-        pkt.data, av_free);
+        gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY, pkt->data,
+        pkt->size, 0, pkt->size, pkt, gst_ffmpegaudenc_free_avpacket);
 
     codec = ffmpegaudenc->context->codec;
-    if ((codec->capabilities & CODEC_CAP_VARIABLE_FRAME_SIZE)) {
+    if ((codec->capabilities & CODEC_CAP_VARIABLE_FRAME_SIZE) || !buffer) {
+      /* FIXME: Not really correct, as -1 means "all the samples we got
+         given so far", which may not be true depending on the codec,
+         but we have no way to know AFAICT */
       ret = gst_audio_encoder_finish_frame (enc, outbuf, -1);
     } else {
-      ret = gst_audio_encoder_finish_frame (enc, outbuf, frame.nb_samples);
+      ret = gst_audio_encoder_finish_frame (enc, outbuf, nsamples);
     }
   } else {
     GST_LOG_OBJECT (ffmpegaudenc, "no output produced");
+    g_slice_free (AVPacket, pkt);
     ret = GST_FLOW_OK;
   }
 
@@ -567,7 +627,7 @@ gst_ffmpegaudenc_drain (GstFFMpegAudEnc * ffmpegaudenc)
     do {
       GstFlowReturn ret;
 
-      ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, NULL, 0, &have_data);
+      ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, NULL, &have_data);
       if (ret != GST_FLOW_OK || have_data == 0)
         break;
     } while (try++ < 10);
@@ -578,10 +638,7 @@ static GstFlowReturn
 gst_ffmpegaudenc_handle_frame (GstAudioEncoder * encoder, GstBuffer * inbuf)
 {
   GstFFMpegAudEnc *ffmpegaudenc;
-  gsize size;
   GstFlowReturn ret;
-  guint8 *in_data;
-  GstMapInfo map;
   gint have_data;
 
   ffmpegaudenc = (GstFFMpegAudEnc *) encoder;
@@ -610,12 +667,7 @@ gst_ffmpegaudenc_handle_frame (GstAudioEncoder * encoder, GstBuffer * inbuf)
         info->channels, info->position, ffmpegaudenc->ffmpeg_layout);
   }
 
-  gst_buffer_map (inbuf, &map, GST_MAP_READ);
-  in_data = map.data;
-  size = map.size;
-  ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, in_data, size, &have_data);
-  gst_buffer_unmap (inbuf, &map);
-  gst_buffer_unref (inbuf);
+  ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, inbuf, &have_data);
 
   if (ret != GST_FLOW_OK)
     goto push_failed;
