@@ -173,13 +173,10 @@ gst_ffmpeg_idct_algo_get_type (void)
       {FF_IDCT_SIMPLEMMX, "Simple MMX", "simplemmx"},
       {FF_IDCT_ARM, "ARM", "arm"},
       {FF_IDCT_ALTIVEC, "Altivec", "altivec"},
-      {FF_IDCT_SH4, "SH4", "sh4"},
       {FF_IDCT_SIMPLEARM, "Simple ARM", "simplearm"},
-      {FF_IDCT_IPP, "IPP", "ipp"},
       {FF_IDCT_XVID, "XVID", "xvid"},
       {FF_IDCT_SIMPLEARMV5TE, "Simple ARMV5TE", "simplearmv5te"},
       {FF_IDCT_SIMPLEARMV6, "Simple ARMV6", "simplearmv6"},
-      {FF_IDCT_SIMPLEVIS, "Simple Vis", "simplevis"},
       {FF_IDCT_FAAN, "FAAN", "faan"},
       {FF_IDCT_SIMPLENEON, "Simple NEON", "simpleneon"},
       {0, NULL, NULL},
@@ -516,13 +513,19 @@ gst_ffmpeg_cfg_init (void)
   gst_ffmpeg_add_pspec (pspec, config.rc_buffer_aggressivity, FALSE, mpeg,
       NULL);
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT (57, 3, 0)
   pspec = g_param_spec_int ("rc-max-rate", "Ratecontrol Maximum Bitrate",
       "Ratecontrol Maximum Bitrate", 0, G_MAXINT, 0,
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+#else
+  pspec = g_param_spec_int64 ("rc-max-rate", "Ratecontrol Maximum Bitrate",
+      "Ratecontrol Maximum Bitrate", 0, G_MAXINT64, 0,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+#endif
   gst_ffmpeg_add_pspec (pspec, config.rc_max_rate, FALSE, mpeg, NULL);
 
-  pspec = g_param_spec_int ("rc-min-rate", "Ratecontrol Minimum Bitrate",
-      "Ratecontrol Minimum Bitrate", 0, G_MAXINT, 0,
+  pspec = g_param_spec_int64 ("rc-min-rate", "Ratecontrol Minimum Bitrate",
+      "Ratecontrol Minimum Bitrate", 0, G_MAXINT64, 0,
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   gst_ffmpeg_add_pspec (pspec, config.rc_min_rate, FALSE, mpeg, NULL);
 
@@ -665,7 +668,7 @@ gst_ffmpeg_cfg_init (void)
   gst_ffmpeg_add_pspec (pspec, interlaced, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("max-bframes", "Max B-Frames",
-      "Maximum B-frames in a row", 0, FF_MAX_B_FRAMES, 0,
+      "Maximum B-frames in a row", 0, INT_MAX, 0,
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   gst_ffmpeg_add_pspec (pspec, config.max_b_frames, FALSE, mpeg, NULL);
 
@@ -773,6 +776,15 @@ gst_ffmpeg_cfg_install_property (GstFFMpegVidEncClass * klass, guint base)
               : pint->default_value, pspec->flags);
           break;
         }
+        case G_TYPE_INT64:{
+          GParamSpecInt64 *pint = G_PARAM_SPEC_INT64 (pspec);
+
+          pspec = g_param_spec_int64 (name, nick, blurb,
+              pint->minimum, pint->maximum,
+              lavc_default ? G_STRUCT_MEMBER (gint64, ctx, ctx_offset)
+              : pint->default_value, pspec->flags);
+          break;
+        }
         case G_TYPE_UINT:{
           GParamSpecUInt *puint = G_PARAM_SPEC_UINT (pspec);
 
@@ -865,6 +877,11 @@ gst_ffmpeg_cfg_set_property (GObject * object,
       G_STRUCT_MEMBER (gint, ffmpegenc, qdata->offset) =
           g_value_get_int (value);
       break;
+    case G_TYPE_INT64:
+      g_return_val_if_fail (qdata->size == sizeof (gint64), TRUE);
+      G_STRUCT_MEMBER (gint64, ffmpegenc, qdata->offset) =
+          g_value_get_int64 (value);
+      break;
     case G_TYPE_FLOAT:
       g_return_val_if_fail (qdata->size == sizeof (gfloat), TRUE);
       G_STRUCT_MEMBER (gfloat, ffmpegenc, qdata->offset) =
@@ -927,6 +944,11 @@ gst_ffmpeg_cfg_get_property (GObject * object,
       g_return_val_if_fail (qdata->size == sizeof (gint), TRUE);
       g_value_set_int (value, G_STRUCT_MEMBER (gint, ffmpegenc, qdata->offset));
       break;
+    case G_TYPE_INT64:
+      g_return_val_if_fail (qdata->size == sizeof (gint64), TRUE);
+      g_value_set_int64 (value, G_STRUCT_MEMBER (gint64, ffmpegenc,
+              qdata->offset));
+      break;
     case G_TYPE_FLOAT:
       g_return_val_if_fail (qdata->size == sizeof (gfloat), TRUE);
       g_value_set_float (value,
