@@ -41,7 +41,7 @@ struct _GstFFMpegVidDec
   gboolean opened;
 
   /* current output pictures */
-  enum PixelFormat pic_pix_fmt;
+  enum AVPixelFormat pic_pix_fmt;
   gint pic_width;
   gint pic_height;
   gint pic_par_n;
@@ -70,7 +70,7 @@ struct _GstFFMpegVidDec
   GstBufferPool *internal_pool;
   gint pool_width;
   gint pool_height;
-  enum PixelFormat pool_format;
+  enum AVPixelFormat pool_format;
   GstVideoInfo pool_info;
 };
 
