@@ -290,7 +290,7 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   GstCaps *allowed_caps;
   GstCaps *icaps;
   GstVideoCodecState *output_format;
-  enum PixelFormat pix_fmt;
+  enum AVPixelFormat pix_fmt;
   GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
   GstFFMpegVidEncClass *oclass =
       (GstFFMpegVidEncClass *) G_OBJECT_GET_CLASS (ffmpegenc);
@@ -643,6 +643,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
     }
   }
 
+  ffmpegenc->picture->format = ffmpegenc->context->pix_fmt;
+  ffmpegenc->picture->width = GST_VIDEO_FRAME_WIDTH (&buffer_info->vframe);
+  ffmpegenc->picture->height = GST_VIDEO_FRAME_HEIGHT (&buffer_info->vframe);
+
   ffmpegenc->picture->pts =
       gst_ffmpeg_time_gst_to_ff (frame->pts /
       ffmpegenc->context->ticks_per_frame, ffmpegenc->context->time_base);
@@ -685,12 +689,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
       pkt->size, 0, pkt->size, pkt, gst_ffmpegvidenc_free_avpacket);
   frame->output_buffer = outbuf;
 
-  /* buggy codec may not set coded_frame */
-  if (ffmpegenc->context->coded_frame) {
-    if (ffmpegenc->context->coded_frame->key_frame)
-      GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
-  } else
-    GST_WARNING_OBJECT (ffmpegenc, "codec did not provide keyframe info");
+  if (pkt->flags & AV_PKT_FLAG_KEY)
+    GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
+  else
+    GST_VIDEO_CODEC_FRAME_UNSET_SYNC_POINT (frame);
 
   return gst_video_encoder_finish_frame (encoder, frame);
 
@@ -756,8 +758,10 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
           pkt->size, 0, pkt->size, pkt, gst_ffmpegvidenc_free_avpacket);
       frame->output_buffer = outbuf;
 
-      if (ffmpegenc->context->coded_frame->key_frame)
+      if (pkt->flags & AV_PKT_FLAG_KEY)
         GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
+      else
+        GST_VIDEO_CODEC_FRAME_UNSET_SYNC_POINT (frame);
 
       flow_ret =
           gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
